#ifndef SCIQLOPPLOTS_BINDINGS_H
#define SCIQLOPPLOTS_BINDINGS_H
#define QT_ANNOTATE_ACCESS_SPECIFIER(a) __attribute__((annotate(#a)))
#include <memory>
#include "numpy_wrappers.hpp"


#include <SciQLopPlots/plot.hpp>
#include <SciQLopPlots/graph.hpp>
#include <SciQLopPlots/sync_panel.hpp>
#include <SciQLopPlots/axis_range.hpp>
#include <SciQLopPlots/time_span.hpp>
#include <SciQLopPlots/Interfaces/IPlotWidget.hpp>
#include <SciQLopPlots/Interfaces/IGraph.hpp>

#endif // SCIQLOPPLOTS_BINDINGS_H
