/*------------------------------------------------------------------------------
-- This file is a part of the SciQLop Software
-- Copyright (C) 2022, Plasma Physics Laboratory - CNRS
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-------------------------------------------------------------------------------*/
/*-- Author : Alexis Jeandet
-- Mail : alexis.jeandet@member.fsf.org
----------------------------------------------------------------------------*/
#pragma once
#include "GraphicObject.hpp"
#include "SciQLopPlots/axis_range.hpp"
#include <QObject>

namespace SciQLopPlots::interfaces
{
class ITimeSpan : public QObject, public GraphicObject
{
    Q_OBJECT

public:
    ITimeSpan(IPlotWidget* plot) : GraphicObject { plot, enums::Layers::Shapes } { }
    virtual ~ITimeSpan();
    inline virtual void set_range(const axis::range& time_range)
    {
        (void) time_range;
        throw std::runtime_error { "You must implement ITimeSpan::set_range" };
    }

    inline virtual axis::range range() const
    {
        throw std::runtime_error { "You must implement ITimeSpan::set_range" };
    }

    Q_SIGNAL void range_changed(axis::range new_time_range);
};

}
